indexing
   description: "Object that handles the specials file"
   author: "Steven Hanley"

class
   
   SPECIALS
      
creation
   make
   
feature {NONE} -- Creation
   
   make is
      once
         read_specials
      end

feature {ANY} -- Queries
   
   character: CHARACTER
    -- the character we converted
   
   success: BOOLEAN
    -- was there an error

   convert (s: STRING) is
      -- store the character representation of the string
      require
         string_not_void: s /= Void
         string_not_empty: not s.is_empty
      local
         i: INTEGER
      do
         s.to_upper
         success := True
         if s.is_integer then -- this is the number already, you can have &#NNN;
            i := s.to_integer
            if i < 0 or else i > 255 then -- check the number is in range
               success := False
            else
               character := i.to_character
            end
         elseif data.has (s) then
            i := data.at (s)
            character := i.to_character
         else
            success := False
         end
      end
   
feature {NONE} -- Private
   
   data: DICTIONARY[INTEGER, STRING] is
      -- read in words and numbers
      once
         !!Result.with_capacity (256)
      end
   
   filename: STRING is "eiffel/specials.txt"
    -- file containing the list
   
   read_specials is
    -- read the contents of the file into the dictionary
      local
         file: STD_FILE_READ
         i, lc: INTEGER
         w: STRING
         a: ARRAY[STRING]
      do
         if file_exists (filename) then
            !!file.connect_to (filename)
            if file.is_connected then
               from 
                  -- nothing
                -- until file.end_of_input loop
                -- this did not work at all so I will give up and 
                -- hard code the number of lines in the file for now
                until lc = 100 loop  
                  file.read_line
                  
                  -- either mechanism below for retrieving the two 
                  -- words will work, was trying both to see if it 
                  -- would stop the problem with end_of_input not 
                  -- working, neither did
                  
                  --file.read_word
                  --w := clone(file.last_string)
                  --file.read_integer
                  --i := clone(file.last_integer).item
                  a := file.last_string.split
                  i := a.item(a.lower+1).to_integer
                  w := a.item(a.lower)
                  w.to_upper
                  
                  data.put (i, w)
                  --io.put_string (lc.to_string + ": " + w + " " + i.to_string + "%N")
                  lc := lc + 1
               end
               file.disconnect
            else
               std_error.put_string (filename + " could not be opened for reading%N")
               die_with_code (1) -- error stop executing, non zero return
            end
         else
            std_error.put_string (filename + " does not exist%N")
            die_with_code (2) -- error stop executing, non zero return
         end
      end
   
end -- class SPECIALS
