#! /usr/bin/perl -w

use strict;
use Time::HiRes qw (gettimeofday);

$| = 1;

my (@t1,@t2);

# starts a timing
sub start_time {
   @t1 = gettimeofday;
}

# ends a timing and retuns the time difference in seconds
sub end_time {
   @t2 = gettimeofday;

   return ((($t2[0] - $t1[0])*1000000) + ($t2[1]-$t1[1]))/1000000;
}

sub sum (@) {
   my (@arr) = @_;
   my $total = 0;
   foreach my $val (@arr) {
      $total += $val;
   }
   return $total;
}

if (! -e "made") {
   print "You should run make to create the executables and test data first.\n";
   exit (1);
}

my $lf = "large_garbled_file";
my $sf = "small_garbled_file";

$ENV{"CLASSPATH"} = $ENV{"CLASSPATH"}.":./java";

my $lfs = -s $lf;
my $sfs = -s $sf;

my @desc = ( "Fast C",         "Slow C", 
             "Fast Eiffel",        "Slow Eiffel", 
             "Fast Java",        "Perl Regexp");

my @progs = ("./c/fast_entity", "./c/slow_entity", 
             "./eiffel/fast_entity", "./eiffel/slow_entity", 
             "java fast_entity", "./perl/perl_entity");

for my $i (0..$#desc) {
   print "Testing $desc[$i]\n";
   my @stimes;
   my @ltimes;
   for my $lc (0..2) {
      my $srunstring = "$progs[$i] < $sf > /dev/null";
      my $lrunstring = "$progs[$i] < $lf > /dev/null";

      start_time();
      `$srunstring`;
      $stimes[$lc] = end_time();
      start_time();
      `$lrunstring`;
      $ltimes[$lc] = end_time();
   }
   my $sres = join ('   ', map { sprintf("%.4f", $_)." s ".
                                 sprintf("%.4f", (($sfs/$_)/1024))." KB/s\n" 
                               } @stimes);
   my $lres = join ('   ', map { sprintf("%.4f", $_)." s ".
                                 sprintf("%.4f", (($lfs/$_)/1024))." KB/s\n" 
                               } @ltimes);
   my $sav = sprintf ("%.4f s %.4f KB/s", 
                      sum (@stimes)/@stimes,
                      sum (map { ($sfs/$_)/1024 } @stimes)/@ltimes);
   my $lav = sprintf ("%.4f s %.4f KB/s", 
                      sum (@ltimes)/@ltimes,
                      sum (map { ($lfs/$_)/1024 } @ltimes)/@ltimes);

   print "Small Data Set\n   ", $sres, "Average $sav\n";
   print "Large Data Set\n   ", $lres, "Average $lav\n\n";
}


